; IDL DISLIN wrapper procedure
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin, func_name, a, b, c, d, e, f, g, h, i, j, k, l, return=result, type=return_type, debug=debug

	common dislin_config, DISLIN_PATH, DISLIN_LIB, DISLIN_FILE
	common dislin_skip_config, dislin_skip_page

	if (keyword_set (dislin_skip_page)) then return

	DISLIN_DEFAULT_PATHS = [ getenv ("DISLIN"), getenv ("HOME")+"/dislin", "/usr/local/dislin", "/opt/local/dislin" ]
	DISLIN_DEFAULT_LIBS = [ "/libdislin.so", "/libdislin.dylib" ]

	; Uncomment for extensive debugging:
	; debug = 1

	func_name = strlowcase (strtrim (func_name, 2))
	num_params = n_params ()

	; Set user-defined DISLIN library
	if (func_name eq 'setlib') then begin
		if (num_params lt 2) then begin
			print, "ERROR: Please provide a filename for your DISLIN library."
			stop
		end
		if (size (a, /type) ne 7) then begin
			print, "ERROR: Please provide a string as parameter for a filename."
			stop
		end
		DISLIN_LIB = a
		DISLIN_FILE = ""
		return
	end

	; Set user-defined path to DISLIN installation
	if (func_name eq 'setpath') then begin
		if (num_params lt 2) then begin
			print, "ERROR: Please provide a path to your DISLIN installation."
			stop
		end
		if (size (a, /type) ne 7) then begin
			print, "ERROR: Please provide a string as parameter for a path."
			stop
		end
		DISLIN_DEFAULT_PATHS = a
		DISLIN_PATH = ""
	end

	; Find DISLIN installation
	if (size (DISLIN_PATH, /type) ne 7) then DISLIN_PATH = ""
	if (size (DISLIN_LIB, /type) ne 7) then DISLIN_LIB = DISLIN_DEFAULT_LIBS
	if (DISLIN_PATH eq "") then begin
		num_found = 0
		for pos = 0, n_elements (DISLIN_LIB)-1 do begin
			found = where (file_test (DISLIN_DEFAULT_PATHS + DISLIN_LIB[pos]), num_found)
			if (num_found ge 1) then break
		end
		if (num_found le 0) then begin
			print, "ERROR: No DISLIN installation found at the given path(s)."
			print, "DISLIN_DEFAULT_PATHS: ", DISLIN_DEFAULT_PATHS
			stop
		end
		DISLIN_PATH = DISLIN_DEFAULT_PATHS[found]
		DISLIN_LIB = DISLIN_DEFAULT_LIBS[pos]
		DISLIN_FILE = ""
	end
	if (DISLIN_FILE eq "") then begin
		num_found = 0
		for pos = 0, n_elements (DISLIN_LIB)-1 do begin
			found = where (file_test (DISLIN_PATH + DISLIN_LIB[pos]), num_found)
			if (num_found ge 1) then break
		end
		DISLIN_LIB = DISLIN_LIB[pos]
		DISLIN_FILE = DISLIN_PATH + DISLIN_LIB
		if (func_name eq 'setpath') then return
	end

	; Mixed by-reference and by-value calls
	if (num_params ge 2) then begin
		value = bytarr (num_params - 1)
		value[*] = 1
	end
	if (size (a, /type) ne 0) then if (size (a, /n_dim) gt 0) then value[0] = 0
	if (size (b, /type) ne 0) then if (size (b, /n_dim) gt 0) then value[1] = 0
	if (size (c, /type) ne 0) then if (size (c, /n_dim) gt 0) then value[2] = 0
	if (size (d, /type) ne 0) then if (size (d, /n_dim) gt 0) then value[3] = 0
	if (size (e, /type) ne 0) then if (size (e, /n_dim) gt 0) then value[4] = 0
	if (size (f, /type) ne 0) then if (size (f, /n_dim) gt 0) then value[5] = 0
	if (size (g, /type) ne 0) then if (size (g, /n_dim) gt 0) then value[6] = 0
	if (size (h, /type) ne 0) then if (size (h, /n_dim) gt 0) then value[7] = 0
	if (size (i, /type) ne 0) then if (size (i, /n_dim) gt 0) then value[8] = 0
	if (size (j, /type) ne 0) then if (size (j, /n_dim) gt 0) then value[9] = 0
	if (size (k, /type) ne 0) then if (size (k, /n_dim) gt 0) then value[10] = 0
	if (size (l, /type) ne 0) then if (size (l, /n_dim) gt 0) then value[11] = 0

	; Special calls containing by-reference scalar parameters
	case func_name of
		'gaxpar': begin
			value[4:8] = 0
			e = 0.0
			f = 0.0
			g = 0.0
			h = 0.0
			i = 0
		end
		'getpag': begin
			value[0:1] = 0
			a = 0
			b = 0
		end
		'getor': begin
			value[0:1] = 0
			a = 0
			b = 0
		end
		'getpos': begin
			value[0:1] = 0
			a = 0
			b = 0
		end
		'getlen': begin
			value[0:2] = 0
			a = 0
			b = 0
			c = 0
		end
		'gmxalf': begin
			value[1:2] = 0
			b = ""
			c = ""
		end
		'getdig': begin
			value[0:2] = 0
			a = 0
			b = 0
			c = 0
		end
		'getgrf': begin
			value[0:3] = 0
			a = 0.0
			b = 0.0
			c = 0.0
			d = 0.0
		end
		'gettic': begin
			value[0:2] = 0
			a = 0
			b = 0
			c = 0
		end
		'gettcl': begin
			value[0:1] = 0
			a = 0
			b = 0
		end
		'getsp1': begin
			value[0:2] = 0
			a = 0
			b = 0
			c = 0
		end
		'getsp2': begin
			value[0:2] = 0
			a = 0
			b = 0
			c = 0
		end
		'getscl': begin
			value[0:2] = 0
			a = 0
			b = 0
			c = 0
		end
		'getscm': begin
			value[0:2] = 0
			a = 0
			b = 0
			c = 0
		end
		'getlab': begin
			value[0:2] = 0
			a = ""
			b = ""
			c = ""
		end
		'getsym': begin
			value[0:1] = 0
			a = 0
			b = 0
		end
		'getres': begin
			value[0:1] = 0
			a = 0
			b = 0
		end
		'getind': begin
			value[1:3] = 0
			b = 0.0
			c = 0.0
			d = 0.0
		end
		'getrgb': begin
			value[0:2] = 0
			a = 0.0
			b = 0.0
			c = 0.0
		end
		'getscr': begin
			value[0:1] = 0
			a = 0
			b = 0
		end
		'getran': begin
			value[0:1] = 0
			a = 0
			b = 0
		end
		'getvk': begin
			value[0:2] = 0
			a = 0
			b = 0
			c = 0
		end
		'getwin': begin
			value[0:3] = 0
			a = 0
			b = 0
			c = 0
			d = 0
		end
		'getclp': begin
			value[0:3] = 0
			a = 0
			b = 0
			c = 0
			d = 0
		end
		else:
; *** WORK HERE:
; add all getter functions from
; www2.mps.mpg.de/dislin/kap7.html
	end

	; Special return values
	; 3 = LONG
	; 4 = FLOAT
	; 5 = DOUBLE
	; 7 = STRING
	; See description of the "SIZE" function in the IDL help.
	if (size (return_type, /type) ne 0) then begin
		if (size (return_type, /type) eq 7) then begin
			return_type = strlowcase (strtrim (return_type, 2))
			case return_type of
				'long':		return_type = 3
				'float':	return_type = 4
				'double':	return_type = 5
				'string':	return_type = 7
				else: begin
					print, "ERROR: The given return type '"+return_type+"' is unknown or not allowed."
					stop
				end
			end
		end
		if (size (return_type, /type) ne 2) then return_type = -1
		if ((return_type lt 3) or (return_type gt 7) or (return_type eq 6)) then begin
			print, "ERROR: The given return type '"+strtrim (return_type, 2)+"' is unknown or not allowed."
			stop
		end
	end

	; Initializations
	switch func_name of
		'legini': begin
				a = string (" ", format='(A'+strtrim (b*c, 2)+')')
				break
			end
		'nlmess':
		'pdfbuf':
		'piegrf':
		'rbfpng': begin
				if (size (a, /type) eq 0) then a = ""
				break
			end
	end

	; Convert all integer parameters to long (DISLIN expects 32 bit integers)
	if (size (a, /type) eq 2) then a = long (a)
	if (size (b, /type) eq 2) then b = long (b)
	if (size (c, /type) eq 2) then c = long (c)
	if (size (d, /type) eq 2) then d = long (d)
	if (size (e, /type) eq 2) then e = long (e)
	if (size (f, /type) eq 2) then f = long (f)
	if (size (g, /type) eq 2) then g = long (g)
	if (size (h, /type) eq 2) then h = long (h)
	if (size (i, /type) eq 2) then i = long (i)
	if (size (j, /type) eq 2) then j = long (j)
	if (size (k, /type) eq 2) then k = long (k)
	if (size (l, /type) eq 2) then l = long (l)

	; Convert all double parameters to float (DISLIN expects 32 bit floats)
	if (size (a, /type) eq 5) then a = float (a)
	if (size (b, /type) eq 5) then b = float (b)
	if (size (c, /type) eq 5) then c = float (c)
	if (size (d, /type) eq 5) then d = float (d)
	if (size (e, /type) eq 5) then e = float (e)
	if (size (f, /type) eq 5) then f = float (f)
	if (size (g, /type) eq 5) then g = float (g)
	if (size (h, /type) eq 5) then h = float (h)
	if (size (i, /type) eq 5) then i = float (i)
	if (size (j, /type) eq 5) then j = float (j)
	if (size (k, /type) eq 5) then k = float (k)
	if (size (l, /type) eq 5) then l = float (l)

	; Convert all multi-dimensional arrays from IDL- to C-order (DISLIN expects memory-alignment in the last dimension)
	if (size (a, /type) ne 0) then if (size (a, /n_dim) ge 2) then a = transpose (a)
	if (size (b, /type) ne 0) then if (size (b, /n_dim) ge 2) then b = transpose (b)
	if (size (c, /type) ne 0) then if (size (c, /n_dim) ge 2) then c = transpose (c)
	if (size (d, /type) ne 0) then if (size (d, /n_dim) ge 2) then d = transpose (d)
	if (size (e, /type) ne 0) then if (size (e, /n_dim) ge 2) then e = transpose (e)
	if (size (f, /type) ne 0) then if (size (f, /n_dim) ge 2) then f = transpose (f)
	if (size (g, /type) ne 0) then if (size (g, /n_dim) ge 2) then g = transpose (g)
	if (size (h, /type) ne 0) then if (size (h, /n_dim) ge 2) then h = transpose (h)
	if (size (i, /type) ne 0) then if (size (i, /n_dim) ge 2) then i = transpose (i)
	if (size (j, /type) ne 0) then if (size (j, /n_dim) ge 2) then j = transpose (j)
	if (size (k, /type) ne 0) then if (size (k, /n_dim) ge 2) then k = transpose (k)
	if (size (l, /type) ne 0) then if (size (l, /n_dim) ge 2) then l = transpose (l)

	all_value = (size (value, /type) eq 0)

	if (keyword_set (debug)) then begin
		print, "DISLIN, '"+func_name+"', "+strtrim (num_params, 2)
		if (num_params ge 2) then print, "A: ", a
		if (num_params ge 3) then print, "B: ", b
		if (num_params ge 4) then print, "C: ", c
		if (num_params ge 5) then print, "D: ", d
		if (num_params ge 6) then print, "E: ", e
		if (num_params ge 7) then print, "F: ", f
		if (num_params ge 8) then print, "G: ", g
		if (num_params ge 9) then print, "H: ", h
		if (num_params ge 10) then print, "I: ", i
		if (num_params ge 11) then print, "J: ", j
		if (num_params ge 12) then print, "K: ", k
		if (num_params ge 13) then print, "L: ", l
		if (keyword_set (return_type)) then print, "RETURN: ", return_type
	end

	case num_params of
		1:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name)
		2:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a)
		3:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b)
		4:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c)
		5:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d)
		6:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d, e)
		7:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d, e, f)
		8:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d, e, f, g)
		9:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d, e, f, g, h)
		10:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d, e, f, g, h, i)
		11:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d, e, f, g, h, i, j)
		12:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d, e, f, g, h, i, j, k)
		13:	result = call_external (DISLIN_FILE, /AUTO_GLUE, ALL_VALUE=all_value, VALUE=value, RETURN_TYPE=return_type, func_name, a, b, c, d, e, f, g, h, i, j, k, l)
	end

	; Convert all multi-dimensional arrays back from C- to IDL-order
	if (size (a, /type) ne 0) then if (size (a, /n_dim) ge 2) then a = transpose (a)
	if (size (b, /type) ne 0) then if (size (b, /n_dim) ge 2) then b = transpose (b)
	if (size (c, /type) ne 0) then if (size (c, /n_dim) ge 2) then c = transpose (c)
	if (size (d, /type) ne 0) then if (size (d, /n_dim) ge 2) then d = transpose (d)
	if (size (e, /type) ne 0) then if (size (e, /n_dim) ge 2) then e = transpose (e)
	if (size (f, /type) ne 0) then if (size (f, /n_dim) ge 2) then f = transpose (f)
	if (size (g, /type) ne 0) then if (size (g, /n_dim) ge 2) then g = transpose (g)
	if (size (h, /type) ne 0) then if (size (h, /n_dim) ge 2) then h = transpose (h)
	if (size (i, /type) ne 0) then if (size (i, /n_dim) ge 2) then i = transpose (i)
	if (size (j, /type) ne 0) then if (size (j, /n_dim) ge 2) then j = transpose (j)
	if (size (k, /type) ne 0) then if (size (k, /n_dim) ge 2) then k = transpose (k)
	if (size (l, /type) ne 0) then if (size (l, /n_dim) ge 2) then l = transpose (l)

	; Storage allocation
	switch func_name of
		'pdfbuf':
		'rbfpng': begin
				a = string (" ", format='(A'+strtrim (result, 2)+')')
				break
			end
	end
end

