; IDL DISLIN wrapper shortcut procedure to create a new axis system
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_axis, left, top, plot_size, type=type, bar=bar, title=title, text_size=text_size, labels=labels, labeling=labeling, label_type=label_type, label_axis=label_axis, no_label=no_label, column=column, row=row, distance=distance, line_width=line_width, frame_color=frame_color, frame_width=frame_width, bar_pos=bar_pos, bar_width=bar_width, bar_distance=bar_distance, bar_area=bar_area, bar_frame=bar_frame, color=color, resolution=resolution, draw_background=draw_background, conserve=conserve, offset=offset, target=target

	common dislin_page_config, default_text_font, default_text_size, default_tex_mode, default_tex_italics, default_hardware_font
	common dislin_axis_config, default_type, default_label_type, default_label_axis, default_offset, default_target
	common dislin_skip_config, dislin_skip_page

	if (keyword_set (dislin_skip_page)) then return

	if ((size (type, /type) ne 7) and keyword_set (type)) then type = '3D'
	if (size (type, /type) ne 7) then type = ''
	is_2D = not strcmp (type, '3D', /fold_case)
	bar = keyword_set (bar) or keyword_set (bar_pos) or keyword_set (bar_width) or keyword_set (bar_distance) or keyword_set (bar_area) or keyword_set (bar_frame) or (is_2D and (n_elements (labels) ge 3))
	if (n_elements (plot_size) eq 1) then plot_size = [ plot_size, plot_size ]
	if (n_elements (resolution) eq 1) then resolution = [ resolution, resolution ]
	if (size (column, /type) eq 0) then column = 0
	if (size (row, /type) eq 0) then row = 0
	if (size (distance, /type) eq 0) then distance = [ left, top ]
	if (n_elements (distance) eq 1) then distance = [ distance, distance ]
	if (size (labeling, /type) eq 0) then labeling = [ "name", "name", "ticks", "ticks" ]
	if (n_elements (labeling) eq 1) then labeling = replicate (labeling, 4)
	dislin, 'setgrf', labeling[0], labeling[1], labeling[2], labeling[3]
	if (total (strcmp (labeling, 'none', /fold_case)) eq 4) then dislin, 'nograf'
	if (size (label_type, /type) eq 0) then if (is_2D) then label_type = "vert" else label_type = "standard"
	if (size (label_axis, /type) eq 0) then label_axis = "y"

	dislin, 'solid'
	offset = round ([ left+column*long (plot_size[0]+distance[0]), top+plot_size[1]+row*long (plot_size[1]+distance[1]) ])
	target = plot_size
	dislin, 'axspos', offset[0], offset[1]
	dislin, 'axslen', round (plot_size[0]), round (plot_size[1])
	if (size (line_width, /type) ne 0) then dislin, 'linwid', round (line_width)
	if (n_elements (labels) ge 1) then begin
		direction = [ 'x', 'y', 'z' ]
		for pos = 0, n_elements (labels) - 1 do dislin, 'name', labels[pos], direction[pos]
	end

	if (size (frame_color, /type) ne 0) then dislin, 'frmclr', round (frame_color)
	if (size (frame_width, /type) ne 0) then dislin, 'frame', round (frame_width)
	if (keyword_set (bar)) then begin
		if (not keyword_set (bar_pos)) then bar_pos = 'FIXED'
		if (size (bar_width, /type) eq 0) then bar_width = 85
		default_bar_distance = bar_width
		if (size (bar_distance, /type) eq 0) then bar_distance = default_bar_distance
		if (size (bar_frame, /type) eq 0) then bar_frame = 0
		bar_pos = strupcase (strtrim (bar_pos, 2))
		bar_space = bar_distance + bar_width + 2 * bar_frame + default_text_size[2]
		if (n_elements (labels) ge 3) then bar_space += default_text_size[2]
		if (size (bar_area, /type) eq 0) then bar_area = bar_space
		if (n_elements (bar_area) eq 1) then bar_area = replicate (bar_area, 2)
		if (bar_pos eq 'TOP') then begin
			bar_area[0] = 0
			bar_area[1] += default_text_size[2]
			bar_distance -= default_text_size[2]
			dislin, 'labtyp', "hori", "z"
			bar_size = plot_size[0]
		end else begin
			bar_area[1] = 0
			label_axis += "z"
			bar_size = plot_size[1]
		end
		dislin, 'posbar', bar_pos
		dislin, 'widbar', round (bar_width)
		dislin, 'spcbar', round (bar_distance)
		dislin, 'frmbar', round (bar_frame)
		if (keyword_set (conserve)) then begin
			dislin, 'ax3len', round (plot_size[0]), round (plot_size[1]), round (bar_size)
		end else begin
			dislin, 'ax3len', round (plot_size[0]-bar_area[0]), round (plot_size[1]-bar_area[1]), round (bar_size)
			target -= bar_area
		end
	end else begin
		dislin, 'nobar'
		if (is_2D) then dislin, 'labels', "none", "z"
	end

	if (size (no_label, /type) eq 7) then begin
		axes = [ 'x', 'y', 'z' ]
		for pos = 0, n_elements (no_label)-1 do begin
			if (keyword_set (no_label[pos])) then dislin, 'axends', no_label[pos], axes[pos]
		end
	end

	default_type = type
	if (strcmp (label_type, "horizontal", /fold_case)) then label_type = "hori"
	if (strcmp (label_type, "vertical", /fold_case)) then label_type = "vert"
	default_label_type = label_type
	default_label_axis = label_axis
	default_offset = offset
	default_target = target

	if (size (color, /type) ne 0) then dislin_color, color

	if (n_elements (resolution) eq 2) then dislin, 'autres', round (resolution[0]), round (resolution[1])
	if (size (draw_background, /type) eq 0) then draw_background = 1
	if (not keyword_set (draw_background)) then dislin, 'nobgd'

	if (keyword_set (text_size)) then begin
		dislin, 'htitle', round (text_size[0])
		if (n_elements (text_size) ge 2) then dislin, 'height', round (text_size[1]) else dislin, 'height', round (text_size[0])
		if (n_elements (text_size) ge 3) then dislin, 'hname', round (text_size[2]) else dislin, 'hname', round (text_size[0])
	end

	dislin_title, title, /save
end

