; IDL DISLIN wrapper shortcut procedure to create a bar chart plot
; Version 1.4, 24.02.2016, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_bar, position, data, bs, be, wx, wy, type=type, horizontal=horizontal, width=width, border=border, pattern=pattern, group=group, spacing=spacing, change=change, color=color, text_size=text_size, front_color=front_color, side_color=side_color, top_color=top_color, label_color=label_color, label_position=label_position, label_type=label_type, line=line

	common dislin_axis_config, default_type, default_label_type, default_label_axis, default_offset, default_target

	NaN = !Values.D_NaN
	num_params = n_params ()

	if (num_params lt 1) then begin
		print, 'WARNING: nothing plotted because no data is given.'
		return
	end
	pos = position
	elements = n_elements (pos)
	if (num_params eq 1) then begin
		data = position
		pos = 1 + dindgen (elements)
	end
	if (num_params le 2) then begin
		bar_start = replicate (0.0, elements)
		bar_end = data
	end else if (num_params eq 3) then begin
		bar_end = bs
		bar_start = data
	end else begin
		if (size (wx, /type) eq 0) then wx = replicate (0.5, elements)
		if (size (wy, /type) eq 0) then wy = replicate (0.5, elements)
		bar_start = bs
		bar_end = be
		if ((size (color, /type) ne 0) and (size (color, /type) ne 7)) then bar_colors = color
		if (size (bar_colors, /type) eq 0) then bar_colors = replicate (-1, elements)
		data_is_3D = 1
		if (n_elements (wx) ne elements) then begin
			print, 'WARNING: nothing plotted because WX array differs in size: ', elements, n_elements (wx)
			return
		end
		if (n_elements (wy) ne elements) then begin
			print, 'WARNING: nothing plotted because WY array differs in size: ', elements, n_elements (wy)
			return
		end
		if (n_elements (bar_colors) ne elements) then begin
			print, 'WARNING: nothing plotted because COLOR array differs in size: ', elements, n_elements (bar_colors)
			return
		end
	end
	if ((num_params ge 2) and (n_elements (data) ne elements)) then begin
		print, 'WARNING: nothing plotted because DATA array differs in size: ', elements, n_elements (data)
		return
	end
	if (n_elements (bar_start) ne elements) then begin
		print, 'WARNING: nothing plotted because BS array differs in size: ', elements, n_elements (bar_start)
		return
	end
	if (n_elements (bar_end) ne elements) then begin
		print, 'WARNING: nothing plotted because BE array differs in size: ', elements, n_elements (bar_end)
		return
	end
	if (size (type, /type) eq 0) then type = default_type
	if (size (type, /type) ne 7) then type = ''
	if ((type eq '') or strcmp (type, '3D', /fold_case)) then begin
		if (keyword_set (horizontal)) then type += 'hori' else type += 'vert'
	end
	if (size (spacing, /type) eq 0) then spacing = 0
	if (size (change, /type) eq 0) then change = 0
	if ((size (change, /type) ne 7) and keyword_set (change)) then change = 'pattern'

	num_colors = n_elements (color)
	scalar_color = (size (color, /type) eq 7) or ((size (color, /type) ne 4) and (size (color, /type) ne 5))
	if ((scalar_color and (num_colors eq 1)) or (not scalar_color and (num_colors eq 3))) then dislin_color, color
	if (n_elements (text_size) eq 1) then if (text_size eq 1) then dislin, 'reset', 'height' else dislin, 'height', round (text_size)
	if (n_elements (text_size) ge 2) then dislin, 'height', round (text_size[1])

	if (size (front_color, /type) ne 0) then begin
		num_colors = n_elements (front_color)
		if (not keyword_set (change)) then change = 'none'
		if ((size (type, /type) ne 7) and ((size (side_color, /type) ne 0) or (size (side_color, /type) ne 0))) then type = '3D'
		if (size (side_color, /type) eq 0) then side_color = front_color
		if (num_colors ne n_elements (side_color)) then begin
			print, 'WARNING: nothing plotted because COLOR and SIDE_COLOR have to have the same size: ', num_colors, n_elements (side_color)
			return
		end
		if (size (top_color, /type) eq 0) then top_color = front_color
		if (num_colors ne n_elements (top_color)) then begin
			print, 'WARNING: nothing plotted because COLOR and TOP_COLOR have to have the same size: ', num_colors, n_elements (top_color)
			return
		end
		dislin, 'barclr', round (front_color), round (side_color), round (top_color), num_colors
	end

	if (n_elements (line) eq 1) then begin
		if (size (line, /type) ne 7) then line = 'solid'
		dislin, line
	end else if (n_elements (line) ge 2) then begin
		dislin, 'myline', round (line), n_elements (line)
	end

	if (size (change, /type) eq 7) then dislin, 'chnbar', change
	num_patterns = n_elements (pattern)
	if (num_patterns eq 1) then dislin, 'shdpat', round (pattern)
	if (size (pattern, /n_dimensions) eq 1) then begin
		for pos = 0, num_patterns-1 do dislin, 'patcyc', round (pattern[pos])
	end
	if (size (group, /type) ne 0) then dislin, 'bargrp', round (group), double (spacing)
	if (size (width, /type) ne 0) then dislin, 'barwth', width
	if (size (border, /type) ne 0) then dislin, 'barbor', round (border)

	if (size (label_color, /type) ne 0) then dislin, 'labclr', round (label_color), 'bars'
	if (size (label_type, /type) eq 7) then dislin, 'labels', label_type, 'bars'
	if (size (label_position, /type) eq 7) then dislin, 'labpos', label_position, 'bars'
	if (size (label_digits, /type) ne 0) then dislin, 'labdig', round (label_digits), 'bars'
	if (size (type, /type) eq 7) then dislin, 'bartyp', type

	if (keyword_set (data_is_3D)) then begin
		dislin, 'bars3d', pos, data, bar_start, bar_end, wx, wy, bar_colors, elements
	end else begin
		dislin, 'bars', pos, bar_start, bar_end, elements
	end

end

