; IDL DISLIN wrapper shortcut procedure to create a new errorbar plot
; Version 1.6, 16.03.2017, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_errorbar, x, y, error, above=above, below=below, color=color, symbol=symbol, size=size, line=line, mode=mode, horizontal=horizontal, width=width, legend=legend, title=title

	if (n_elements (width) eq 1) then dislin, 'linwid', round (width)
	if (n_elements (mode) eq 1) then dislin, 'incmrk', round (mode)
	if (n_elements (symbol) eq 1) then dislin, 'marker', round (symbol)
	if (n_elements (size) eq 1) then dislin, 'hsymbl', round (size)
	if (n_elements (line) eq 1) then begin
		if (size (line, /type) ne 7) then line = 'solid'
		dislin, line
	end else if (n_elements (line) ge 2) then begin
		dislin, 'myline', round (line), n_elements (line)
	end
	bar_type = "vert"
	if (keyword_set (horizontal)) then bar_type = "hori"

	if (size (color, /type) ne 0) then dislin_color, color

	elements = n_elements (x)
	if (elements eq 0) then begin
		print, 'WARNING: nothing plotted because X array is empty.'
		return
	end

	if (n_elements (y) ne elements) then begin
		print, 'WARNING: nothing plotted because X and Y arrays differ in size: ', elements, n_elements (y)
		return
	end
	if (size (error, /type) ne 0) then begin
		if (n_elements (error[*,0]) ne elements) then begin
			print, 'WARNING: nothing plotted because error array differs in size: ', elements, n_elements (error)
			return
		end
		if (size (error, /n_dimensions) eq 1) then begin
			above = error * 0.5
			below = above
		end else if ((size (error, /dimensions))[1] eq 2) then begin
			above = error[*,0]
			below = error[*,1]
		end else begin
			print, 'WARNING: nothing plotted because error array is of unknown format.'
			return
		end
	end else if (keyword_set (above) and keyword_set (below)) then begin
		error = above + below
	end else begin
		print, 'WARNING: nothing plotted because no error arrays are given.'
		return
	end
	if (n_elements (above) ne elements) then begin
		print, 'WARNING: nothing plotted because above array differs in size: ', elements, n_elements (above)
		return
	end
	if (n_elements (below) ne elements) then begin
		print, 'WARNING: nothing plotted because below array differs in size: ', elements, n_elements (above)
		return
	end

	good = where (finite (x) and finite (y) and finite (above) and finite (below), num_good)
	if (num_good eq 0) then begin
		print, 'WARNING: nothing plotted because all data is denormal.'
		return
	end

	if (size (title, /type) ne 7) then title = ''
	if (title ne '') then begin
		if (size (legend, /type) ne 7) then legend = title else legend = [ legend, title ]
	end

	dislin, 'bartyp', bar_type

	dislin, 'errbar', double (x[good]), double (y[good]), double (below[good]), double (above[good]), num_good
end

