; IDL DISLIN wrapper shortcut procedure to create a new legend
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_legend, legend, pos=pos, title=title, length=length, select=select, margin=margin, spacing=spacing, text_size=text_size, symbol=symbol, line=line, width=width, frame_width=frame_width, color=color, background=background, xs=xs, xe=xe, ys=ys, ye=ye, center_x=center_x, center_y=center_y, cleanup=cleanup, keep=keep

	common dislin_skip_config, dislin_skip_page

	if (keyword_set (dislin_skip_page)) then return

	NaN = !Values.D_NaN

	num = n_elements (legend)
	if (num lt 1) then begin
		print, 'WARNING: nothing plotted because legend array is empty.'
		return
	end

	if (n_elements (title) ne 1) then title = ''
	if (size (length, /type) eq 0) then length = 4.0
	if (size (margin, /type) eq 0) then margin = 0.5
	if (size (spacing, /type) eq 0) then spacing = 1.0
	if (size (symbol, /type) eq 0) then symbol = 1.0
	if (size (center_x, /type) eq 0) then center_x = 0.5
	if (size (center_y, /type) eq 0) then center_y = 0.5

	if (n_elements (width) eq 1) then dislin, 'linwid', round (width)
	if (n_elements (frame_width) eq 1) then dislin, 'frame', round (frame_width)
	if (n_elements (line) eq 1) then begin
		if (size (line, /type) ne 7) then line = 'solid'
		dislin, line
	end else if (n_elements (line) ge 2) then begin
		dislin, 'myline', round (line), n_elements (line)
	end

	if (size (color, /type) ne 0) then dislin_color, color

	dislin, 'legini', buffer, num, max (strlen (legend))
	if (size (background, /type) ne 0) then dislin, 'legbgd', round (background)
	dislin, 'legopt', double (length), double (margin), double (spacing)

	dislin, 'legtit', title
	if (n_elements (text_size) eq 1) then if (text_size eq 1) then dislin, 'reset', 'height' else dislin, 'height', round (text_size)
	if (n_elements (text_size) ge 2) then dislin, 'height', round (text_size[1])
	for i = 1, n_elements (legend) do begin
		index = i
		dislin, 'leglin', buffer, legend[i-1], index
	end
	if (keyword_set (cleanup) or ((size (cleanup, /type) eq 0) and not keyword_set (keep))) then dummy = size (temporary (legend), /type)
	dislin, 'legval', symbol, 'symbol'

	if (size (pos, /type) eq 7) then begin
		if (pos eq 'page-upper-left') then begin
			pos = 1
		end else if (pos eq 'page-upper-right') then begin
			pos = 2
		end else if (pos eq 'page-lower-right') then begin
			pos = 3
		end else if (pos eq 'page-lower-left') then begin
			pos = 4
		end else if (pos eq 'bottom-left') then begin
			pos = 5
		end else if (pos eq 'bottom-right') then begin
			pos = 6
		end else if (pos eq 'top-right') then begin
			pos = 7
		end else if (pos eq 'top-left') then begin
			pos = 8
		end else begin
			if ((size (xs, /type) eq 0) or (size (xe, /type) eq 0) or (size (ys, /type) eq 0) or (size (ye, /type) eq 0)) then begin
				message, "ERROR: need the parameters 'xs','xe','ys','ye' for position '"+pos+"'!"
			end
			distance_percentage = 0.02
			dislin, 'nxlegn', buffer, return=box_x, type='long'
			dislin, 'nylegn', buffer, return=box_y, type='long'
			dislin, 'nxposn', float (xs), return=plot_xs, type='long'
			dislin, 'nyposn', float (ys), return=plot_ys, type='long'
			dislin, 'xinvrs', round (plot_xs + box_x), return=leg_x, type='float'
			dislin, 'yinvrs', round (plot_ys - box_y), return=leg_y, type='float'
			x_dist = distance_percentage * (xe - xs)
			y_dist = distance_percentage * (ye - ys)
			dislin, 'nxposn', xs + x_dist, return=left, type='long'
			dislin, 'nxposn', xe - x_dist - (leg_x - xs), return=right, type='long'
			dislin, 'nyposn', ye - y_dist, return=upper, type='long'
			dislin, 'nyposn', ys + y_dist + (leg_y - ys), return=lower, type='long'
			dislin, 'nxposn', xs + center_x * (xe - xs - (leg_x - xs)), return=center, type='long'
			dislin, 'nyposn', ys + center_y * (ye - ys - (leg_y - ys)), return=middle, type='long'
			if (pos eq 'bottom-center') then begin
				pos = [ center, lower ]
			end else if (pos eq 'top-center') then begin
				pos = [ center, upper ]
			end else if (pos eq 'middle-right') then begin
				pos = [ right, middle ]
			end else if (pos eq 'middle-left') then begin
				pos = [ left, middle ]
			end else if (pos eq 'middle-center') then begin
				pos = [ center, middle ]
			end else begin
				pos = NaN
			end
		end
	end
	if (size (pos, /type) eq 0) then pos = NaN
	if (finite (pos[0], /NaN)) then pos = 8
	if (n_elements (pos) eq 2) then dislin, 'legpos', round (pos[0]), round (pos[1])
	if (n_elements (select) ge 1) then dislin, 'legsel', round (select), n_elements (select)

	dislin, 'legend', buffer, round (pos[0])
end

