; IDL DISLIN wrapper shortcut procedure to draw a straight line
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_line, data_x, data_y, log=log, slope=slope, reference=reference, label=label, length=length, delta=delta, size=size, line=line, width=width, mode=mode, interpolate=interpolate, color=color, legend=legend, title=title, arrow=arrow, ratio=ratio, opening=opening
	if (size (log, /type) eq 0) then log = ''
	if (size (log, /type) ne 7) then if (keyword_set (log)) then log = 'xy' else log = ''
	if (size (reference, /type) eq 0) then reference = 0
	if (n_elements (reference) eq 1) then reference = [ reference, reference ]
	if (size (label, /type) ne 7) then if (keyword_set (label)) then label = 'slope = <slope>' else label = ''
	if (size (line, /type) eq 0) then line = 'solid'
	if (size (mode, /type) eq 0) then mode = 0
	if (not keyword_set (color)) then color = 'black'
	if ((n_elements (color) ne 3) and (size (color, /type) ne 7)) then color = 'blue'

	if ((size (data_x, /n_dimensions) eq 2) and (size (data_y, /type) eq 0)) then begin
		data_y = reform (data_x[*,1])
		data_x = reform (data_x[*,0])
	end

	x = data_x
	y = data_y
	if (keyword_set (interpolate) and (n_elements (x) eq 2) and (n_elements (y) eq 1)) then begin
		if (size (slope, /type) eq 0) then slope = 1.0
		xs = x[0]
		xe = x[1]
		ys = y[0]
		x = dindgen (ceil ((xe - xs) > 1) + 1) + xs
		y = ys + (x - xs) * slope
	end
	if (strpos (log, 'x') ge 0) then x = 10^x
	if (strpos (log, 'y') ge 0) then y = 10^y

	num = n_elements (x)
	if (not keyword_set (mode) and (num ge 2) and (size (title, /type) ne 7)) then begin
		if (size (width, /type) ne 0) then dislin, 'linwid', round (width)
		if (n_elements (line) eq 1) then begin
			if (size (line, /type) ne 7) then line = 'solid'
			dislin, line
		end else if (n_elements (line) ge 2) then begin
			dislin, 'myline', round (line), n_elements (line)
		end
		dislin_color, color
		for pos = 0, num-2 do begin
			x1 = double (x[pos])
			y1 = double (y[pos])
			x2 = double (x[pos+1])
			y2 = double (y[pos+1])
			dislin, 'rline', x1, y1, x2, y2
		end
	end else begin
		dislin_curve, x, y, mode=mode, color=color, line=line, width=width, legend=legend, title=title
	end

	if (keyword_set (label)) then begin
		ends_x = float (x[[0,num-1]])
		ends_y = float (y[[0,num-1]])
		dislin, 'trfrel', ends_x, ends_y, 2
		angle = atan (-(ends_y[1] - ends_y[0]) / (ends_x[1] - ends_x[0]))
		slope_label = '<slope>'
		pos = strpos (label, slope_label)
		if (pos ge 0) then begin
			if (size (length, /type) eq 0) then length = 5
			tmp_length = length > 1
			tmp_digits = (length - 2) > 0
			if (tmp_digits le 0) then slope = round (slope) else slope = round (slope * 10.0^tmp_digits) / 10.0^tmp_digits
			if (slope lt 0.0) then tmp_length++
			slope_str = strmid (strtrim (slope, 2), 0, tmp_length)
			label = strmid (label, 0, pos) + slope_str + strmid (label, pos+strlen (slope_label))
		end

		dislin, 'angle', round (180/!DPi * angle)

		if (size (delta, /type) eq 0) then delta = 0
		if (n_elements (delta) eq 1) then delta = [ delta, 0 ]
		shift_x = -delta[1] * sin (angle) + delta[0] * cos (angle)
		shift_y = -delta[1] * cos (angle) - delta[0] * sin (angle)
		if (n_elements (size) ge 1) then begin
			dislin, 'height', round (size[0])
			shift_x += -size[0] * sin (angle) * 1.5
			shift_y += -size[0] * cos (angle) * 1.5
		end

		pos_x = [ x[reference[0]] ]
		pos_y = [ y[reference[1]] ]
		dislin, 'trfrel', pos_x, pos_y, 1
		dislin, 'messag', label, round (pos_x[0] + shift_x), round (pos_y[0] + shift_y)
		dislin, 'reset', "angle"
	end

	if (keyword_set (arrow) or keyword_set (ratio) or keyword_set (opening)) then begin
		if (size (arrow, /type) eq 7) then begin
			reverse = (strlowcase (arrow) eq 'reverse')
		end else if (keyword_set (arrow)) then begin
			reverse = (arrow lt 0)
		end
		dislin_arrow, x[0], y[0], x[num-1]-x[0], y[num-1]-y[0], log=log, line=line, width=width, color=color, ratio=ratio, opening=opening, reverse=reverse, /bare
	end
end

